@@ -1,11 +1,38 @@
 import time
 import random
+import base64 as b64
 from typing import Optional, Union, List
+from abc import ABC, abstractmethod
 
-from .display import color
+from .utils import color
 
 
-class Plain:
+class Element(ABC):
+    """消息元素基类"""
+
+    @abstractmethod
+    def to_json(self):
+        """将消息对象转换为 json"""
+
+    @staticmethod
+    @abstractmethod
+    def from_json(json: dict):
+        """将 json 转换为消息对象"""
+
+    @classmethod
+    def subclasses_str(cls):
+        """返回所有子类名字符串的列表"""
+        return [c.__name__ for c in cls.__subclasses__()]
+
+    @classmethod
+    def _handle_base64(cls, base64: Optional[Union[bytes, str]]) -> Optional[bytes]:
+        if isinstance(base64, str):
+            return b64.b64encode(open(base64, 'rb').read())
+        else:
+            return base64
+
+
+class Plain(Element):
     def __init__(self, text):
         self.text = text
 
@@ -27,7 +54,7 @@ class Plain:
         return Plain(text)
 
 
-class At:
+class At(Element):
     def __init__(self, qq, display: str = None):
         self.qq = qq
         self.display = display
@@ -54,7 +81,7 @@ class At:
         return At(qq, display)
 
 
-class AtAll:
+class AtAll(Element):
     def __init__(self):
         pass
 
@@ -67,8 +94,7 @@ class AtAll:
         else:
             return False
 
-    @staticmethod
-    def to_json():
+    def to_json(self):
         return {'type': 'AtAll'}
 
     @staticmethod
@@ -76,7 +102,7 @@ class AtAll:
         return AtAll()
 
 
-class Face:
+class Face(Element):
     def __init__(self, face_id: int = None, name: str = None):
         self.face_id = face_id
         self.name = name
@@ -116,14 +142,18 @@ class Face:
         return Face(name=name)
 
 
-class Image:
-    def __init__(self, path: str = None, url: str = None, image_id: str = None):
+class Image(Element):
+    def __init__(self, path: str = None, url: str = None, image_id: str = None,
+                 base64: Optional[Union[bytes, str]] = None):
         self.path = path
         self.url = url
         self.image_id = image_id
+        self.base64 = Element._handle_base64(base64)
 
     def __repr__(self):
-        if not self.url:
+        if not self.image_id:
+            return '[Image]'
+        elif not self.url:
             return f'[Image:{self.image_id}]'
         else:
             return f'[Image:{self.image_id} | {self.url}]'
@@ -141,6 +171,8 @@ class Image:
             return {'type': 'Image', 'url': self.url}
         elif self.image_id:
             return {'type': 'Image', 'imageId': self.image_id}
+        elif self.base64:
+            return {'type': 'Image', 'base64': self.base64.decode('utf-8')}
 
     @property
     def is_flash(self) -> bool:
@@ -171,18 +203,26 @@ class Image:
     def from_id(image_id: str) -> 'Image':
         return Image(image_id=image_id)
 
+    @staticmethod
+    def from_base64(base64) -> 'Image':
+        return Image(base64=Element._handle_base64(base64))
 
-class FlashImage:
-    def __init__(self, path: str = None, url: str = None, image_id: str = None):
+
+class FlashImage(Element):
+    def __init__(self, path: str = None, url: str = None, image_id: str = None,
+                 base64: Optional[Union[bytes, str]] = None):
         self.path = path
         self.url = url
         self.image_id = image_id
+        self.base64 = Element._handle_base64(base64)
 
     def __repr__(self):
-        if not self.url:
-            return f'[Image:{self.image_id}]'
+        if not self.image_id:
+            return '[FlashImage]'
+        elif not self.url:
+            return f'[FlashImage:{self.image_id}]'
         else:
-            return f'[Image:{self.image_id} | {self.url}]'
+            return f'[FlashImage:{self.image_id} | {self.url}]'
 
     def __eq__(self, other):
         if isinstance(other, FlashImage):
@@ -197,6 +237,8 @@ class FlashImage:
             return {'type': 'FlashImage', 'url': self.url}
         elif self.image_id:
             return {'type': 'FlashImage', 'imageId': self.image_id}
+        elif self.base64:
+            return {'type': 'FlashImage', 'base64': self.base64.decode('utf-8')}
 
     @property
     def is_flash(self) -> bool:
@@ -227,15 +269,23 @@ class FlashImage:
     def from_id(image_id: str) -> 'FlashImage':
         return FlashImage(image_id=image_id)
 
+    @staticmethod
+    def from_base64(base64) -> 'FlashImage':
+        return FlashImage(base64=Element._handle_base64(base64))
 
-class Voice:
-    def __init__(self, path: str = None, url: str = None, voice_id: str = None):
+
+class Voice(Element):
+    def __init__(self, path: str = None, url: str = None, voice_id: str = None,
+                 base64: Optional[Union[bytes, str]] = None):
         self.path = path
         self.url = url
         self.voice_id = voice_id
+        self.base64 = Element._handle_base64(base64)
 
     def __repr__(self):
-        if not self.url:
+        if not self.voice_id:
+            return '[Voice]'
+        elif not self.url:
             return f'[Voice:{self.voice_id}]'
         else:
             return f'[Voice:{self.voice_id} | {self.url}]'
@@ -253,6 +303,8 @@ class Voice:
             return {'type': 'Voice', 'url': self.url}
         elif self.voice_id:
             return {'type': 'Voice', 'voiceId': self.voice_id}
+        elif self.base64:
+            return {'type': 'Voice', 'base64': self.base64.decode('utf-8')}
 
     @staticmethod
     def from_json(json: dict) -> 'Voice':
@@ -273,8 +325,12 @@ class Voice:
     def from_id(voice_id: str) -> 'Voice':
         return Voice(voice_id=voice_id)
 
+    @staticmethod
+    def from_base64(base64) -> 'Voice':
+        return Voice(base64=Element._handle_base64(base64))
 
-class Xml:
+
+class Xml(Element):
     def __init__(self, xml):
         self.xml = xml
 
@@ -296,7 +352,7 @@ class Xml:
         return Xml(xml=xml)
 
 
-class Json:
+class Json(Element):
     def __init__(self, json):
         self.json = json
 
@@ -318,7 +374,7 @@ class Json:
         return Json(json=json)
 
 
-class App:
+class App(Element):
     def __init__(self, content):
         self.content = content
 
@@ -340,7 +396,29 @@ class App:
         return App(content=json)
 
 
-class Dice:
+class Poke(Element):
+    def __init__(self, name: str):
+        self.name = name
+
+    def __repr__(self):
+        return f'[Poke:{self.name}]'
+
+    def __eq__(self, other):
+        if isinstance(other, Poke):
+            return self.name == other.name
+        else:
+            return False
+
+    def to_json(self):
+        return {'type': 'Poke', 'name': self.name}
+
+    @staticmethod
+    def from_json(json: dict):
+        name = json.get('name', None)
+        return Poke(name=name)
+
+
+class Dice(Element):
     def __init__(self, value: Optional[int] = None):
         self.value = value if value else random.randint(1, 6)
 
@@ -362,44 +440,94 @@ class Dice:
         return Dice(value=value)
 
 
-class File:
-    def __init__(self, file_id: str, internal_id: int, name: str, size: int):
+class File(Element):
+    def __init__(self, file_id: str, name: str, size: int):
         self.file_id = file_id
-        self.internal_id = internal_id
         self.name = name
         self.size = size
 
     def __repr__(self):
         return f'[File:{self.name} | {self.file_id}]'
 
+    def __eq__(self, other):
+        if isinstance(other, File):
+            return self.file_id == other.file_id and self.name == other.name and self.size == other.size
+        else:
+            return False
+
+    def to_json(self):
+        return {'type': 'File', 'id': self.file_id, 'name': self.name, 'size': self.size}
+
     @staticmethod
     def from_json(json: dict):
         file_id = json.get('id', None)
-        internal_id = json.get('internalId', None)
         name = json.get('name', None)
         size = json.get('size', None)
-        return File(file_id=file_id, internal_id=internal_id, name=name, size=size)
+        return File(file_id=file_id, name=name, size=size)
+
+
+class MiraiCode(Element):
+    def __init__(self, code: str):
+        self.code = code
+
+    def __repr__(self):
+        return f'[MiraiCode:{self.code}]'
+
+    def __eq__(self, other):
+        if isinstance(other, MiraiCode):
+            return self.code == other.code
+        else:
+            return False
+
+    def to_json(self):
+        return {'type': 'MiraiCode', 'code': self.code}
+
+    @staticmethod
+    def from_json(json: dict):
+        code = json.get('code', None)
+        return MiraiCode(code=code)
+
+
+class BotMessage:
+    """bot 发出的消息"""
+
+    def __init__(self, msg_chain: List, msg_type: str = None, msg_id: int = None, target: int = None):
+        self.chain = msg_chain
+        self.msg_type = msg_type
+        self.id = msg_id
+        self.text = ''
+        self.target = target
+        for ele in self.chain:
+            if ele['type'] in Element.subclasses_str():
+                self.text += eval(ele['type']).from_json(ele).__repr__()
+
+    def __repr__(self):
+        return f"{time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())} {self.msg_type} #{self.id} - " \
+               f"{self.target} <- {self.text.__repr__()}"
 
 
 class Message:
-    def __init__(self, msg: dict):
+    """消息基类"""
+
+    def __init__(self, msg: dict, bot_qq: int):
         self.json = msg
+        self._bot_qq = bot_qq
         msg_chain = msg.get('messageChain', None)
         try:
             self.id = msg_chain[0].get('id', None)
         except:
             self.id = None
+        self.time = msg_chain[0].get('time', 0)
         try:
-            self.time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(msg_chain[0].get('time', 0)))
+            self._time = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(self.time))
         except:
-            self.time = None
+            self._time = None
 
         try:
             chain = []
             text = ''
             for ele in msg_chain[1:]:
-                if ele['type'] in ['Plain', 'At', 'AtAll', 'Face', 'Image', 'FlashImage',
-                                   'Voice', 'Xml', 'Json', 'App', 'Dice', 'File']:
+                if ele['type'] in Element.subclasses_str():
                     instance = eval(ele['type']).from_json(ele)
                     chain.append(instance)
                     text += instance.__repr__()
@@ -412,10 +540,13 @@ class Message:
             self.text = ''
 
     def __eq__(self, other):
-        return True if self.chain == other.chain else False
+        if isinstance(other, Message):
+            return True if self.chain == other.chain else False
+        return False
 
     @property
     def plain(self) -> str:
+        """返回消息链中的所有文字"""
         text = ''
         for ele in self.chain:
             if type(ele) == Plain:
@@ -424,6 +555,7 @@ class Message:
 
     @property
     def first_image(self) -> Optional[Union[Image, FlashImage]]:
+        """返回消息链中的第一张图片"""
         for ele in self.chain:
             if type(ele) in [Image, FlashImage]:
                 return ele
@@ -431,19 +563,27 @@ class Message:
 
     @property
     def images(self) -> List[Union[Image, FlashImage]]:
+        """返回消息链中所有图片的列表"""
         return [ele for ele in self.chain if type(ele) in [Image, FlashImage]]
 
     @property
     def file(self) -> Optional[File]:
+        """返回消息链中的文件"""
         for ele in self.chain:
             if isinstance(ele, File):
                 return ele
         return None
 
+    def at_me(self) -> bool:
+        """返回这条消息中是否 at 了 bot"""
+        return any([ele == At(self._bot_qq) for ele in self.chain])
+
 
 class GroupMessage(Message):
-    def __init__(self, msg: dict):
-        super().__init__(msg)
+    """群消息"""
+
+    def __init__(self, msg: dict, bot_qq: int):
+        super().__init__(msg, bot_qq)
         sender = msg.get('sender', {})
         self.sender = sender.get('id', None)
         self.sender_name = sender.get('memberName', None)
@@ -452,25 +592,29 @@ class GroupMessage(Message):
         self.group_name = group.get('name', None)
 
     def __repr__(self):
-        return f'{self.time} GroupMessage #{self.id} {self.group_name}({self.group})' \
+        return f'{self._time} GroupMessage #{self.id} {self.group_name}({self.group})' \
                f' - {self.sender_name}({self.sender}): {self.text.__repr__()}'
 
 
 class FriendMessage(Message):
-    def __init__(self, msg: dict):
-        super().__init__(msg)
+    """好友消息"""
+
+    def __init__(self, msg: dict, bot_qq: int):
+        super().__init__(msg, bot_qq)
         sender = msg.get('sender', {})
         self.sender = sender.get('id', None)
         self.sender_name = sender.get('nickname', None)
 
     def __repr__(self):
-        return f'{self.time} FriendMessage #{self.id}' \
+        return f'{self._time} FriendMessage #{self.id}' \
                f' - {self.sender_name}({self.sender}): {self.text.__repr__()}'
 
 
 class TempMessage(Message):
-    def __init__(self, msg: dict):
-        super().__init__(msg)
+    """群临时消息"""
+
+    def __init__(self, msg: dict, bot_qq: int):
+        super().__init__(msg, bot_qq)
         sender = msg.get('sender', {})
         self.sender = sender.get('id', None)
         self.sender_name = sender.get('memberName', None)
@@ -479,7 +623,7 @@ class TempMessage(Message):
         self.group_name = group.get('name', None)
 
     def __repr__(self):
-        return f'{self.time} TempMessage #{self.id} {self.group_name}({self.group})' \
+        return f'{self._time} TempMessage #{self.id} {self.group_name}({self.group})' \
                f' - {self.sender_name}({self.sender}): {self.text.__repr__()}'
 
 
